# version 1.3
# Copyright (C) 2008 Alban Ramette
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

automaticbinner=function(D){
cat("----------------Automatic binner v.1.3. by A. Ramette----------------\n")
cat('NOTE:   The user needs to set the working directory, as follows:\n')
cat('        for instance:   setwd("c:\\\\R\\\\DIR")\n')
cat('        The result files will be saved in that DIR directory\n')
cat('NOTE:   The user needs to import the D table before starting\n')
cat("  	e.g. D=read.table(\"input.txt\",h=TRUE)\n")
cat("  	D is a table with 3 columns:\n")
cat(" 	D[,1] sample name for each band\n")
cat(" 	D[,2] the second consists of band sizes\n")
cat(" 	D[,3] the last consists of area (fluorescence in absolute value\n")
cat("	Automatic mode... \n")
cat("------------------------------------------------------------\n")

ANS<-readline("Continue? (y/n)....... ")
if(ANS=="y"){

###################### variable declaration
Sm=as.numeric(readline(       "\nLower bound of the size range, e.g. 100:\t"))
SM=as.numeric(readline(         "Higher bound of the size range, e.g. 1000:\t"))
RFIco=as.numeric(readline(      "Minimum RFI cutoff value, e.g. 0.09%:\t\t"))




#cat("\nChecking for existence of bands in the defined peak range:",paste(Sm,"-",SM,sep =""),"...\n")

Checkrange=function(D){
  L=length(unique(D[,1]))#how many samples?
  V=as.vector(D[D[,2]<SM&D[,2]>Sm,1]) #samples that are ok
  VD=as.vector(unique(D[,1]))
  Diff=setdiff(VD,V)
    if(length(Diff)!=0) {
	     cat("Size definition problems! The program was stopped.\n",
       "Samples with a problem:\n")
  	   print(Diff)
	     OK<-0
	   }
  if(length(Diff)==0) {
	 #cat("OK. No problem of sizes\n")
	 OK<-1
   cat("------------------------------------------------------------\n")
   }
  return(OK)
 }#end of the checkrange ########

OK=Checkrange(D)


if(OK==1){

#==================
# 1) range size
Ds<-subset(D,D[,2]>=Sm)
Ds<-subset(Ds,Ds[,2]<=SM)
Ds=Ds[order(Ds[,1]),] #get the right order of sample names

#======calculation of the RFI=========================
	# for each sample, subset the area and calculate its sum (Tarea)
Tarea<-tapply(Ds[,3],Ds[,1],sum)
	# get the list of unique names of samples
SampNames<-names(Tarea)


DsRFI=cbind(Ds,RFI=rep(NA,nrow(Ds)))
for (s in 1:length(SampNames)){
  SU=subset(Ds,Ds[,1]==SampNames[s])
  DsRFI[Ds[,1]==SampNames[s],4]=round(SU[,3]*100/Tarea[s],2)
}


#keep data based on a RFI cutoff
Dfinal<-subset(DsRFI,DsRFI[,4]>=RFIco)

if(length(unique(Dfinal[,1]))!=length(unique(D[,1]))){
cat(" The script was stopped because the chosen RFI cutoff value has removed too many\n") 
cat("peaks for correct computations. Lower the cutoff value, for instance.\n")}
if(length(unique(Dfinal[,1]))==length(unique(D[,1]))){#RFIco check




#RFI table
TableRFI=xtabs(RFI~Dfinal[,1]+Dfinal[,2],Dfinal)

#01Table
Table01=TableRFI
Table01[Table01!=0]<-1


#######################################################

Scan=function(WS,Sh){ #################
          ### Create a vector of binning labels
          #WS      # window size(bp)
          #Sh      # Shift between frames(bp)
          NbF=WS/Sh # Nber of binning Frames
                    
          ##########################
          Shiftpeak=function(Data,Step){####  decrease the column label (peak size) by Step
          	Blist=as.numeric(colnames(Data))
          	Blist_s=Blist-Step
          	Data1=Data
          	colnames(Data1)=Blist_s
          	return(Data1)                  #return a Table with the same content as the original table but with different labels
          }#end of the function shiftpeak ########
           ##########################

          Binner=function(Table=TableRFI,WS,St){
            #for a given TableRFI table, return the binned table
          	#conversion of the column names (already shifted) into bins (fixed bin)

          	 Blist=as.numeric(colnames(Table))
            Convbin=rbind(Peak=Blist,BinNber=rep(NA,length(Blist)),BinStart=rep(NA,length(Blist)))
            #=>3 rows: 1) peak 2) bin Nber  3) Binstart value

            #conversion to bin number for each column
             #E end position vector of the bins
             #S start position vector of the bins
               E=seq(Sm,SM,WS);S=E-WS

               for(j in 1:length(Blist)){
              VAL=Convbin[1,j]
              if(length(which(S<=VAL & VAL<E))==0) {Convbin[2,j]=NA ;Convbin[3,j]=max(E)}
              if(length(which(S<=VAL & VAL<E))!=0) {
                  Convbin[2,j]=which(S<=VAL & VAL<E)  #identifies the bin number
                  Convbin[3,j]=S[Convbin[2,j]]            #identifies the bin starting number
              }
              }
              Convbin[2:3,]=round(Convbin[2:3,],0)
              colnames(Table)=Convbin[3,]   #renaming the colnames with BinStart to prepare for binning

             #for each row, add the columns that have the same row1
          	U1=as.numeric(unique(colnames(Table))) #vector of unique peak sizes
          	U1L=length(U1)
          	W1=matrix(NA,nrow(Table),U1L) #matrix of results with binned bands
          	rownames(W1)=rownames(Table)
          	colnames(W1)=U1

          	for (u in 1:U1L){
          	Dtemp=Table[,colnames(Table)==U1[u]]
          	if(is.null(dim(Dtemp))){W1[,u]=Dtemp}
          	else{
          	W1[,u]=apply(Dtemp,1,sum) #select the columns with the same peak and sum their RFI per line
          	}}

          	return(W1) # binned table, wherein RFI are summed by bin
          }#end of the function Binner#######################
          ##############################################
            #filling the lists

          ShV=seq(0,WS-Sh,Sh) #vector of binning shift values

          TableBinVector=vector("list",NbF)	#to produce the list of Tables of shifted peak labels
          names(TableBinVector)=ShV  #bin names
          for (i in 1:NbF){ #filling the list with the peak relabeled tables
            TableBinVector[[i]]=Shiftpeak(Data=TableRFI,Step=ShV[i])
          }


          BinnedTables=vector("list",NbF)	#to produce the list of Tables of shifted peak labels
          names(BinnedTables)=ShV
          for (i in 1:NbF){ #filling the list with the peak relabeled tables
            BinnedTables[[i]]=Binner(Table=TableBinVector[[i]],WS=WS,St=Sm-ShV[i])    #St (starts)   100 may be changed
          }

          	#to produce a list of sample correlations
          Corlist=lapply(BinnedTables,function(x) as.dist(cor(t(x))))
          MeancorVec=sapply(Corlist,mean)

          Correlations=round(as.numeric(MeancorVec),2)
          NberOTUs=sapply(BinnedTables,ncol)
          

          MaxCorr=as.numeric(which(Correlations==max(Correlations))[1])#which position with the highest corr
                   
          MaxOTUs=as.numeric(which(NberOTUs==max(NberOTUs))[1])#which position with the highest corr

          Summary=rbind(c(Correlations[MaxCorr]," (",NberOTUs[MaxCorr],")"),
                        c(Correlations[MaxOTUs]," (",NberOTUs[MaxOTUs],")")                        
          )
                 
         
           return(Summary)
}#end of Scan##############################


 
RWS=readline("\nWhat is the range of Window Sizes (WS) to be used? e.g. 1 2 5 10 (separated by a space:\t")
RWS= strsplit(RWS," ")[[1]]
RWSl=length(RWS)
SHv=as.numeric(readline("What is the Shift (Sh) value of WS to be used? e.g. 0.1 (only one common value):\t"))

cat("\nplease wait...\n\n")
cat("###### Results ###### \n")
for(i in 1:RWSl){
     #S=Scan(WS=as.numeric(RWS[i]),Sh=SHv)$Summary
     #cat("WS=",RWS[i],"Sh=",SHv,":","\t\t",as.numeric(S[,S[2,]==max(S[2,])]),"\n")
     cat("WS=",RWS[i],", Sh=",SHv," ---------\n")
         S=Scan(WS=as.numeric(RWS[i]),Sh=SHv)
     cat("Highest correlation:\t",S[1,],"\n",sep="")
     cat("Largest OTU number:  \t",S[2,],sep="","\n\n")
     
}

cat("(End of calculations)\n")
 }#RFIco OK
}#OK
}# yes
}#automatic binner